﻿
---------------------------------------------------------
--	Library
---------------------------------------------------------
local bzone = LibStub("LibBabble-Zone-3.0"):GetLookupTable()
local bboss = LibStub("LibBabble-Boss-3.0"):GetLookupTable()
local bzoneRev = LibStub("LibBabble-Zone-3.0"):GetReverseLookupTable()

---------------------------------------------------------
--	Localization
---------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("GridStatusRaidDebuff")
local LGrid = AceLibrary("AceLocale-2.2"):new("Grid")

L:RegisterTranslations("enUS", function()
	return {
	["Raid Debuff"] = true,
		["Option for %s"] = true,
		["Enable"] = true,
		["Enable %s"] = true,		
		["Icon Priority"] = true,
		["Color Priority"] = true,
		["Custom Color"] = true,
		["Color"] = true,
		colorDesc = "Modify Color",		
		["Ignore dispellable debuff"] = true,		
		["Ignore undispellable debuff"] = true,
		["Remained time"] = true,
		["Stackable debuff"] = true,
		["Only color"] = "Only show color, Ignore icon",
		detector = "Debuff detector",
	}
end)

L:RegisterTranslations("koKR", function()
	return {
		["Raid Debuff"] = "레이드 디버프",
		["Option for %s"] = "%s의 설정",
		["Enable"] = "활성화",
		["Enable %s"] = "%s 활성화",		
		["Icon Priority"] = "아이콘 우선순위",
		["Color Priority"] = "색상 우선순위",
		["Custom Color"] = "임의의 색상",
		["Color"] = "색상",
		colorDesc = "색상 변경",
		["Ignore dispellable debuff"] = "해제 가능한 디버프 무시",
		["Ignore undispellable debuff"] = "해제 불가능한 디버프 무시",
		["Remained time"] = "남은 시간",
		["Stackable debuff"] = "중첩 디버프",
		["Only color"] = "색상만 표시",
		detector = "디버프 탐색기",
	}
end)

L:RegisterTranslations("zhCN", function()
	return {
		["Raid Debuff"] = "团队减益",
		["Option for %s"] = "%s 设置",
		["Enable"] = "开启",
		["Enable %s"] = "开启 %s",		
		["Icon Priority"] = "Icon Priority",
		["Color Priority"] = "Color Priority",
		["Custom Color"] = "Custom Color",
		["Color"] = "Color",
		colorDesc = "Modify Color",
		["Ignore dispellable debuff"] = "Ignore dispellable debuff",		
		["Ignore undispellable debuff"] = "Ignore undispellable debuff",
		["Remained time"] = "Remained time",
		["Stackable debuff"] = "Stackable debuff",
		["Only color"] = "Only show color, Ignore icon",
		detector = "Debuff detector",
}
end)

L:RegisterTranslations("zhTW", function()
	return {
		["Raid Debuff"] = "團隊減益",
		["Option for %s"] = "%s 選項",
		["Enable"] = "啟用",
		["Enable %s"] = "啟用%s",
		["Icon Priority"] = "Icon Priority",
		["Color Priority"] = "Color Priority",
		["Custom Color"] = "Custom Color",
		["Color"] = "Color",
		colorDesc = "Modify Color",
		["Ignore dispellable debuff"] = "Ignore dispellable debuff",		
		["Ignore undispellable debuff"] = "Ignore undispellable debuff",
		["Remained time"] = "Remained time",
		["Stackable debuff"] = "Stackable debuff",
		["Only color"] = "Only show color, Ignore icon",
		detector = "Debuff detector",
	}
end)

L:RegisterTranslations("ruRU", function()
	return {
		["Raid Debuff"] = "Дебаффы рейда",
		["Option for %s"] = "Опции %s",
		["Enable"] = "Включить",
		["Enable %s"] = "Включить %s",
		["Icon Priority"] = "Приоритет иконки",
		["Color Priority"] = "Приоритет цвета",
		["Custom Color"] = "Свой цвет",
		["Color"] = "Цвет",
		colorDesc = "Изменение цвета",
		["Ignore dispellable debuff"] = "Игнорировать дебаффы которые можно снять",		
		["Ignore undispellable debuff"] = "Игнорировать дебаффы которые не можно снять",
		["Remained time"] = "Остаток времени",
		["Stackable debuff"] = "Наращиваемые дебаффы",
		["Only color"] = "Показывать только цвет, игнорируя иконку",
		detector = "обнаружение дебаффов",
	}
end)

---------------------------------------------------------
--	local
---------------------------------------------------------
local realzone, detectEnable
local db, myClass, myDispellable
local debuff_list = {}

local colorMap = {
	["Curse"] = { r = .6, g =  0, b = 1},
	["Magic"] = { r = .2, g = .6, b = 1},
	["Poison"] = {r =  0, g = .6, b =  0},
	["Disease"] = { r = .6, g = .4, b =  0},
}

local dispelMap = {
	["PRIEST"] = {["Magic"] = true, ["Disease"] = true},
	["PALADIN"] = {["Magic"] = true, ["Disease"] = true, ["Poison"] = true},
	["MAGE"] = {["Curse"] = true},
	["DRUID"] = {["Curse"] = true, ["Poison"] = true},
	["SHAMAN"] = {["Disease"] = true, ["Poison"] = true, ["Curse"] = (select(5,GetTalentInfo(3,18)) == 1)},	
}


---------------------------------------------------------
--	Core
---------------------------------------------------------
GridStatusRaidDebuff = GridStatus:NewModule("GridStatusRaidDebuff")

local GridStatusRaidDebuff = GridStatusRaidDebuff
GridStatusRaidDebuff.menuName = L["Raid Debuff"]

local GetSpellInfo = GetSpellInfo
local fmt = string.format

GridStatusRaidDebuff.defaultDB = {
	debug = false,	
	isFirst = true,	
	
	["alert_RaidDebuff"] = {
		text = L["Raid Debuff"],
		desc = L["Raid Debuff"],
		enable = true,
		color = { r = .0, g = .0, b = .0, a=1.0 },
		priority = 98,
		range = false,
	},
	
	ignDis = false,
	ignUndis = false,
	
	
	["debuff_options"] = {},
	["custom_debuff"] = {},	
}

function GridStatusRaidDebuff:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatuses()
	db = self.db.profile.debuff_options
end

function GridStatusRaidDebuff:OnEnable()
	self.debugging = self.db.profile.debug		
	
	myClass = select(2, UnitClass("player"))
	myDispellable = dispelMap[myClass]
	
	
	if self.db.profile.isFirst then		
		GridFrame.db.profile.statusmap["icon"].alert_RaidDebuff  =  true,
		GridFrame:UpdateAllFrames()
		self.db.profile.isFirst = false
	end	
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneCheck")
	self:RegisterCustomDebuff()
	
end

function GridStatusRaidDebuff:Reset()
	self.super.Reset(self)
	self:UnregisterStatuses()
	self:RegisterStatuses()
end

function GridStatusRaidDebuff:PLAYER_ENTERING_WORLD()	
	self:ZoneCheck()	
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")	
end

function GridStatusRaidDebuff:ZoneCheck()
	local zonetype
	realzone, zonetype = GetInstanceInfo()

	self:UpdateAllUnit()
	detectEnable = not (zonetype == "none")

	if debuff_list[realzone] then
		if not self:IsEventRegistered("UNIT_AURA") then				
			self:RegisterEvent("UNIT_AURA", "ScanUnit")
			self:RegisterEvent("Grid_UnitJoined")
			if myClass == "SHAMAN" then
				self:RegisterEvent("PLAYER_TALENT_UPDATE")
			end
		end
	else			
		if self:IsEventRegistered("UNIT_AURA") then				
			self:UnregisterEvent("UNIT_AURA")
			self:UnregisterEvent("Grid_UnitJoined")	
			if myClass == "SHAMAN" then
				self:UnregisterEvent("PLAYER_TALENT_UPDATE")
			end
		end
	end
end

function GridStatusRaidDebuff:RegisterStatuses()
	self:RegisterStatus("alert_RaidDebuff", L["Raid Debuff"])
	self:CreateMainMenu()
end

function GridStatusRaidDebuff:UnregisterStatuses()
	self:UnregisterStatus("alert_RaidDebuff")
end

function GridStatusRaidDebuff:Grid_UnitJoined(guid, unitid)
	self:ScanUnit(unitid)
end

function GridStatusRaidDebuff:PLAYER_TALENT_UPDATE()	
	myDispellable["Curse"] = (select(5,GetTalentInfo(3,18)) == 1)	
end

function GridStatusRaidDebuff:UpdateAllUnit()
	for guid, unitid in GridRoster:IterateRoster() do
		self:ScanUnit(unitid)
	end
end

function GridStatusRaidDebuff:ScanUnit(unitid)
	local guid = UnitGUID(unitid)	
	
	if not GridRoster:IsGUIDInRaid(guid) then
		return
	end
	self:Debug("UNIT_AURA", unitid, guid)
	
	local name, rank, icon, count, debuffType, duration, expirationTime, caster, isStealable	
	local settings = self.db.profile["alert_RaidDebuff"]	
	
	if (settings.enable and debuff_list[realzone]) then	
		local d_name, di_prior, dc_prior, dt_prior, d_icon,d_color,d_startTime,d_durTime,d_count
		local data
		
		di_prior = 0
		dc_prior = 0
		dt_prior = 0
		
		local index = 0
		while true do
			index = index + 1
			name, _, icon, count, debuffType, duration, expirationTime, caster, isStealable = UnitAura(unitid, index, "HARMFUL")

			if not name then
				break
			end

			if debuff_list[realzone][name] then		
				data = debuff_list[realzone][name]
				
				if not data.disable and
				   not (self.db.profile.ignDis and myDispellable[debuffType]) and
				   not (self.db.profile.ignUndis and debuffType and not myDispellable[debuffType]) then
				
					if di_prior < data.i_prior then
						di_prior = data.i_prior
						d_name = name
						d_icon = 	not data.noicon and icon
						if data.timer and dt_prior < data.i_prior then
							d_startTime = expirationTime - duration
							d_durTime = duration
						end
					end				
					--Stack
					if data.stackable then
						d_count = count
					end
					--Color Priority
					if dc_prior < data.c_prior then
						dc_prior = data.c_prior					
						d_color = (data.custom_color and data.color) or colorMap[debuffType] or settings.color					
					end
				end
			else
				if false then
					local casterType = UnitClassification(caster)
					if caster and (casterType == "worldboss" or casterType == "elite") then
						self:Debuff(bzoneRev[realzone], name, icon, 5, 5)			
						if not self.db.profile.custom_debuff[realzone][name] then self.db.profile.custom_debuff[realzone][name] = icon end
						if  self.options.args[realzone].args["load zone"].disabled then self:LoadZoneMenu(realzone) end
					end
					self:UpdateAllUnit()
				end
			end
		end
		
		if d_color and not d_color.a then
			d_color.a = settings.color.a
		end
		
		if d_color and d_color.a == 0 then			
			d_color.a = 1
		end
			
		if d_name then
			self.core:SendStatusGained(
			guid, "alert_RaidDebuff", settings.priority, (settings.range and 40),
			d_color, nil, nil, nil, d_icon, d_startTime, d_durTime, d_count)
		else
			self.core:SendStatusLost(guid, "alert_RaidDebuff")	
		end
	else
		self.core:SendStatusLost(guid, "alert_RaidDebuff")	
	end
end

---------------------------------------------------------
--	For External
---------------------------------------------------------
local function getDb(zone, name, arg, ret)	
	if db[zone] and db[zone][name] and db[zone][name][arg] ~= nil then
		return db[zone][name][arg]
	end
	return ret
end

local function insertDb(zone, name, arg, value)
	if not db[zone] then db[zone] = {} end
	if not db[zone][name] then db[zone][name] = {} end
	
	if arg then
		db[zone][name][arg] = value
	end
end

function GridStatusRaidDebuff:Debuff(en_zone, first, second, icon_priority, color_priority, timer, stackable, color, default_disable, noicon)
	local zone = bzone[en_zone]	
	local name, icon, id
	local args, data, order	
	
	self:CreateZoneMenu(zone)	
	
	if type(first) == "number" then
		name, _, icon = GetSpellInfo(first)
		id = first
		order = second		
	else
		name = first
		icon = second
		id = 0
		order = 1000
	end
	
	if not debuff_list[zone][name] then
		debuff_list[zone][name] = {}		
		data = debuff_list[zone][name]
		
		data.debuffId = id
		data.icon = icon
		data.order = order
		data.disable = getDb(zone,name,"disable",default_disable)
		data.i_prior = getDb(zone,name,"i_prior",icon_priority)
		data.c_prior = getDb(zone,name,"c_prior",color_priority)
		data.custom_color = getDb(zone,name,"custom_color",color ~= nil)
		data.color = getDb(zone,name,"color",color)
		data.stackable = getDb(zone,name,"stackable",stackable)
		data.timer = getDb(zone,name,"timer",timer)
		data.noicon = getDb(zone,name,"noicon",noicon)
	end
end


function GridStatusRaidDebuff:BossName(en_zone, order, en_boss)
	local zone = bzone[en_zone]
	local boss = bboss[en_boss]	
	
	self:CreateZoneMenu(zone)
	
	local args = self.options.args
	
	args[zone].args[boss] = {		
			type = "group",
			name = fmt("%s%s%s","   [ ", boss," ]"),
			order = order,
			guiHidden = true,
			args = {}
	}
end

-- Create a custom tooltip for debuff description
local tip = CreateFrame("GameTooltip", "GridStatusRaidDebuffTooltip", nil, "GameTooltipTemplate")
tip:SetOwner(UIParent, "ANCHOR_NONE")
for i = 1, 10 do
	tip[i] = _G["GridStatusRaidDebuffTooltipTextLeft"..i]
	if not tip[i] then
		tip[i] = tip:CreateFontString()
		tip:AddFontStrings(tip[i], tip:CreateFontString())
	end
end

function GridStatusRaidDebuff:LoadZoneMenu(zone)
	local args = self.options.args[zone].args	
	local description, menuName, icon, data	
	local settings = self.db.profile["alert_RaidDebuff"]
	
	if not args["load zone"].disabled then args["load zone"].disabled = true end
	
	for i,k in pairs(args) do
		if k.guiHidden then 
			k.guiHidden = false
		end	
	end
	
	for name,k in pairs(debuff_list[zone]) do
		if not args[name] then
			description = L["Enable %s"]:format(name)
			
			tip:SetHyperlink("spell:"..k.debuffId)
			if tip:NumLines() > 1 then
				description = tip[tip:NumLines()]:GetText()
			end		
			
			menuName = fmt("|T%s:0|t%s", k.icon, name)	
			
			args[name] = {
				type = "group",
				name = menuName,
				desc = description,				
				order = k.order,
				args = {
					["enable"] = {
						type = "toggle",
						name = L["Enable"],
						desc = L["Enable %s"]:format(name),
						order = 1,
						get = function()
										return not k.disable
									end,
						set = function(v)
										insertDb(zone,name,"disable",not v)									
										k.disable = not v
										self:UpdateAllUnit()
									end,
					},
					
					["icon priority"] = {
						type = "range",
						name = L["Icon Priority"],
						desc = L["Option for %s"]:format(L["Icon Priority"]),
						order = 2,
						min = 1,
						max = 10,
						step = 1,
						get = function()									
										return k.i_prior
									end,
						set = function(v)
										insertDb(zone,name,"i_prior",v)
										k.i_prior = v
										self:UpdateAllUnit()
									end,	
					},
					["color priority"] = {
						type = "range",
						name = L["Color Priority"],
						desc = L["Option for %s"]:format(L["Color Priority"]),
						order = 3,
						min = 1,
						max = 10,
						step = 1,
						get = function() 
										return k.c_prior
									end,
						set = function(v)
										insertDb(zone,name,"c_prior",v)
										k.c_prior = v
										self:UpdateAllUnit()
									end,					
					},				
					["Remained time"] = {
						type = "toggle",
						name = L["Remained time"],
						desc = L["Enable %s"]:format(L["Remained time"]),
						order = 4,
						get = function() 
										return k.timer
									end,
						set = function(v)
										insertDb(zone,name,"timer",v)
										k.timer = v
										self:UpdateAllUnit()						
									end,		
					},
					["Stackable debuff"] = {
						type = "toggle",
						name = L["Stackable debuff"],
						desc = L["Enable %s"]:format(L["Stackable debuff"]),
						order = 5,
						get = function() 
										return k.stackable
									end,
						set = function(v)
										insertDb(zone,name,"stackable",v)
										k.stackable = v
										self:UpdateAllUnit()						
									end,					
					},
					["only color"] = {
						type = "toggle",
						name = L["Only color"],
						desc = L["Only color"],
						order = 7,
						get = function() 
										return k.noicon
									end,
						set = function(v)
										insertDb(zone,name,"noicon",v)
										k.noicon = v
										self:UpdateAllUnit()								
									end,	
					},
					["custom color"] = {
						type = "toggle",
						name = L["Custom Color"],
						desc = L["Enable %s"]:format(L["Custom Color"]),
						order = 7,
						get = function() 
										return k.custom_color
									end,
						set = function(v)
										insertDb(zone,name,"custom_color",v)
										k.custom_color = v
										
										if v then
											insertDb(zone,name,"color", settings.color)
											k.color = settings.color										
										end
										
										self:UpdateAllUnit()								
									end,					
					},
					["color"] = {
						type = "color",
						name = L["Color"],
						desc = L["Option for %s"]:format(L["Color"]),
						order = 8,
						disabled = function()
												return not k.custom_color
											end,	
						hasAlpha = false,
						get = function ()									
										t = getDb(zone,name,"color", color or {r = 1, g = 0, b = 0})
										return t.r, t.g, t.b
								  end,
						set = function (ir, ig, ib)
										local t = {r = ir, g = ig, b = ib}
										insertDb(zone,name,"color",t)
									  k.color = t
									  self:UpdateAllUnit()
								  end,
						
					},							
				},
			}
		end
	end
end

function GridStatusRaidDebuff:CreateZoneMenu(zone)
	local args
	if not debuff_list[zone] then 
		debuff_list[zone] = {}
				
		args = self.options.args
					
		args[zone] = {
			type = "group",
			name = zone,
			desc = L["Option for %s"]:format(zone),			
			args = {
				["load zone"] = {
					type = "execute",
					name = "Load",
					desc = "Load",
					func = function()
						self:LoadZoneMenu(zone)
					end,
				}
			
			},
		}
	end
end

function GridStatusRaidDebuff:CreateMainMenu()
	local args = self.options.args	
	
	
	for i,k in pairs(args["alert_RaidDebuff"].args) do
		args[i] = k		
	end	
	
	args["alert_RaidDebuff"].hidden = true
	
	args["Border"] = {
			type = "toggle",
			name = LGrid["Border"],			
			desc = L["Enable %s"]:format(LGrid["Border"]),
			order = 98,
			disabled = InCombatLockdown,
			get = function() return GridFrame.db.profile.statusmap["border"].alert_RaidDebuff end,
			set = function(v) 
							GridFrame.db.profile.statusmap["border"].alert_RaidDebuff  =  v 
							GridFrame:UpdateAllFrames()
						end,
	}	
	args["Icon"] = {
			type = "toggle",
			name = LGrid["Center Icon"],			
			desc = L["Enable %s"]:format(LGrid["Center Icon"]),
			order = 99,
			disabled = InCombatLockdown,
			get = function() return GridFrame.db.profile.statusmap["icon"].alert_RaidDebuff end,
			set = function(v) 
							GridFrame.db.profile.statusmap["icon"].alert_RaidDebuff  =  v 
							GridFrame:UpdateAllFrames()
						end,
	}	
	args["Ignore dispellable"] = {
		type = "toggle",
		name = L["Ignore dispellable debuff"],
		desc = L["Ignore dispellable debuff"],
		order = 100,
		get = function() return self.db.profile.ignDis end,
		set = function(v) 
						self.db.profile.ignDis = v
						self:UpdateAllUnit()
					end,
	
	}	
	args["Ignore undispellable"] = {
		type = "toggle",
		name = L["Ignore undispellable debuff"],
		desc = L["Ignore undispellable debuff"],
		order = 101,
		get = function() return self.db.profile.ignUndis end,
		set = function(v) 
						self.db.profile.ignUndis = v
						self:UpdateAllUnit()
					end,	
	}	

end

function GridStatusRaidDebuff:RegisterCustomDebuff()
	local en_zone
	for zone,j in pairs(self.db.profile.custom_debuff) do		
		en_zone = bzoneRev[zone]
		for name,k in pairs(j) do
			self:Debuff(en_zone, name, k, 5, 5, true, true)			
		end
	end
end

